// comets.h		ver.0.000
//copyright 2016 Beer[Tsutsumi Hiroyuki]

#include	<Be.h>

#if	B_BEOS_VERSION>=0x0520
#include	<Locale.h>
typedef	const char	Char;
#else
#define	_T(str)	str
typedef	char		Char;
#endif

#ifdef	B_HAIKU_VERSION
#define	MENUBAR_HEIGHT	19
#else
#define	MENUBAR_HEIGHT	20
#endif


struct label{
	int		lno;	//line no.
	uint16	adr;
	char	name[9];
};

struct module{
	int		sline,eline;
	int		slbl,elbl;
	int		entl;			//start de shiteishita sakino label
};

struct line{
	char	lbl[9];
	char	opc[7];
	char	opr[240];
	char	rem[256];
};

struct opr{
	char	a[128];
	char	b[128];
	char	c[128];
};

enum flags{
	of=0x01,
	sf=0x02,
	zf=0x04,
};

enum chrset{
	sjis=0x01,
};

#define	MAXSRCSIZE	0x40000
#define	MAXSRCLINE	10000
#define	MAXLABEL	1000
#define	MAXMODULE	100


class comet
{
	public:
		comet();
		~comet();
		void	init(bool *inpf,char *inp,int inpl,bool *outf,char *out,int outl,char *rp);//init sys
		void	reset(char flag);	//init cpu
		void	run(int steps);
		int32	sys_main(void *data);
		int32	cpu_main(void *data);
		void	setfr(int acc);
		void	setfr1(int acc);
		void	setfr2(int acc);
		void	setfr3(int acc,uint16 cy);
		uint16	pr,sp,fr;
		uint16	gr[16];		//8-15...ext
		uint16	mem[0x10000],psize;
		int		svc;		//set -1 when svc-return
		bool	svcw;		//wait svc-complete
		int		job;		//0.init	1.run  -1.end
		int		cset;	//in/out character set
	private:
		thread_id	t_cpu_id,t_sys_id;
		int		stp;
		bool	*inpf,*outf;
		char	*inp,*out;
		int		inpl,outl;
		char	*rp;
		char	flag;
};

class caslassembler
{
	public:
		caslassembler();
		~caslassembler();
		void	init(char *rp,uint16 *mem,uint16 *psize);
		int		assemble(char *src,off_t ssize,char flag);	//init memory,encode,set binary
	private:
		int		encode1();	//step1  pickup label,set adr with label
		int		encode2();	//step2  encode with label   ret.code_bytes
		int		add_lbl(int mno,char *s,int lno,uint16 adr);
		int		search_lbl(int mno,char *s);
		int		split_line(int lno,struct line &l);
		int		split_opr(char *s,struct opr &opr);
		int		enc_adr(int mno,char *s,uint16 *d=NULL);
		int		enc_line(int mno,struct line &l,uint16 *d=NULL);
		int		macro(int mno,struct line &l,uint16 *d=NULL);	//encode macro  ret.bytes (err=-1)
		int		ds(char *s,uint16 *d=NULL);	//encode ds  ret.bytes (err=-1)
		int		dc(int mno,char *s,uint16 *d=NULL);	//encode dc  ret.bytes  mno=-1.literal
	public:
		char	*src;
		off_t	ssize;
		int		lc;		//total line
		int		*slp;	//offset(pointer each line to source)	-1:end
		struct label	lbl[MAXLABEL+1];
		int		lblc;
		struct module	pro[MAXMODULE+1];
		int		proc;	//number of program
		uint16	codeadr[MAXSRCLINE+1];
		char	err[256];	//errmes
	private:
		char	*rp;	//report
		uint16	*mem;
		uint16	*psize;	//program(binary) size
		char	flag;
};



class termview:public BTextView
{
	public:
		termview(BRect frame,BRect textRect);
		~termview();
		void	AttachedToWindow();
//		void	Draw(BRect rect);
		void	Pulse();
		void	KeyDown(const char* bytes,int32 numBytes);
//	virtual	void				MessageReceived(BMessage* message);
		void	puts(char *s);
	public:
//		int		job=0;		//0..none 1..input 2..inputend
		char	instr[256];
		int		inlen;
};

class cometsview:public BView
{
	public:
		cometsview(BRect frame);
		~cometsview();
		void	AttachedToWindow();
		void	Draw(BRect rect);
		void	Pulse();
		void	MouseDown(BPoint pt);
//		void	MouseUp(BPoint pt);
//		void	MouseMoved(BPoint pt, uint32 code, const BMessage *msg);
		void	MessageReceived(BMessage *msg);
//		BMessage*	ddmessage();		//for drag&drop
//		void	ddcopy(BMessage *msg);	//for drag&drop
//		void	select_trans(const char *mime,translator_id *tid,uint32 *fmt);
//		void	redraw(bool rf=true);
//		void	drawframe(const s_sel &fr);
//		void	eraseframe(const s_sel &fr);
		void	load(entry_ref &eref);
		void	reset();
//		void	save(entry_ref &dir,char *name);
	public:
	private:
		BFilePanel	*fp;
		char		title[64];

		char	*src;
		off_t	ssize;
//		caslassembler	cas;
//		comet			cpu;
		int		job;		//0.init  1.load  2.asm  3.stop  4.run  5.svc  6.end
		int		cset;	//in/out character set
		char	flag;		//checkbox  0.asm_report 1.pr_check 2.sp_check 3.ext_mode
		uint16	psize;

		BButton	*b_reset,*b_start,*b_stop,*b_step;
		BCheckBox	*cb_prc,*cb_spc,*cb_opc,*cb_svc,*cb_rpt,*cb_exc,*cb_exh;
	public:		//for terminal
		char	input[256],output[8192];
		bool	inpf,outf;
		char	report[1024];	//from asm/cpu
};

class termwindow:public BWindow
{
	public:
		termwindow(BRect frame);
		bool	QuitRequested();
		termview	*tv;	//terminal(interface)
};

class cometswindow:public BWindow
{
	public:
		cometswindow(BRect frame);
		bool	QuitRequested();
//		void	FrameResized(float width,float height);
//		void	resizeview(int width,int height);
		void	setupmenus();
//		void	get_translator_for_output();
		cometsview	*v;
//		termview	*tv;	//terminal(interface)
		BTextView	*sv;	//status
		BTextView	*rv;	//register
		BScrollBar	*sbv,*sbh;
};

class comets:public BApplication
{
	public:
		comets(int argc,char **argv);
		void	RefsReceived(BMessage *mes);
//		void	MessageReceived(BMessage *mes);
		void	AboutRequested();
		cometswindow	*w;
		termwindow		*tw;
};

caslassembler	*cas;
comet			*cpu;

int32 thread_sys(void *data)
{
	cpu->sys_main(data);
}
int32 thread_cpu(void *data)
{
	cpu->cpu_main(data);
}

const int16	opck1[256]={
	0xff,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,0xf8,0x8f,  -1,  -1,  -1,  -1,0xf8,
	  -1,0xf8,  -1,  -1,  -1,0xf8,  -1,0xf8,  -1,0xf8,  -1,0xf8,  -1,0xf8,  -1,  -1,
	  -1,0xf8,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	0xff,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	  -1,0x88,  -1,0x88,  -1,0x88,  -1,0x88,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	  -1,0x88,  -1,0x88,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88,  -1,  -1,  -1,  -1,  -1,  -1,  -1,  -1,
	0x88,0x88,0x88,0x88,  -1,  -1,  -1,  -1,0x88,0x88,0x88,0x88,0x88,0x88,0x88,0x88
};
